package millikanexperimentapp;

import java.text.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import org.opensourcephysics.display.*;

/**
 * <p>Title: Millikan experiment</p>
 * <p>Description: This is a simulation of the Millikan oil drop experiment.
 * Brownian motion is taken into account.</p>
 * <p>Copybase: Copyright (c) 2005</p>
 * <p>Company: Gymnazium arm. gen. L. Svobodu</p>
 * @author Jakub Dubas, Rudolf Tomori, and Slavomir Tuleja
 * @version 2.7 from February 15, 2005
 */

public class MillikanExperimentApp
    extends JApplet
    implements Runnable {
  private boolean timeRunning = false, voltageon = false, rayon = false;
  int voltage = 0;
  public Graphics g;
  public JPanel plMicroscope = new JPanel();
  public JPanel plControls = new JPanel();
  public JPanel plStopWatch = new JPanel();
  public JPanel plVoltage = new JPanel();
  public JPanel plVoltageBtns = new JPanel();
  public JPanel plOtherControls = new JPanel();
  public JButton btnStart = new JButton("tart");
  public JButton btnReset = new JButton("Vynulova\u0165");
  public JButton btnTurnVoltageOn = new JButton("Zapn\u0165");
  public JButton btnChangePolarity = new JButton("Preplova\u0165");
  public JButton btnInject = new JButton("Vstrekn\u0165");
  public JButton btnTurnRaysOn = new JButton("L\u010De zapn\u0165");
  public JSlider slVoltage = new JSlider();
  public JMenuBar mbMenu = new JMenuBar();
  public JMenu mnHelp = new JMenu("Pomoc");
  DrawingPanel panel = new DrawingPanel();
  double t;
  double dt = 0.01;
  Thread animationThread;
  DecimalFormat decimal = new DecimalFormat("0.0");
  OilDrops drops = new OilDrops();
  GridBagConstraints c = new GridBagConstraints();
  Container cp;

  public void init() {
    TitledBorder title;

    //add panel plMicroscope and set its componensts
    title = BorderFactory.createTitledBorder("Zorn pole mikroskopu");
    plMicroscope.setBorder(title);
    plMicroscope.setLayout(new GridLayout(1, 1));
    plMicroscope.add(panel);
    cp = getContentPane();
    cp.add(plMicroscope, BorderLayout.CENTER);

    //add panel plControls and set its componensts
    title = BorderFactory.createTitledBorder("Ovldanie");
    plControls.setBorder(title);
    cp.add(plControls, BorderLayout.EAST);
    plControls.setLayout(new GridBagLayout());

    //add subpanel plStopWatch and set its componensts
    c.gridx = 0;
    c.gridy = 0;
    c.weighty = 0.0;
    c.anchor = GridBagConstraints.NORTH;
    title = BorderFactory.createTitledBorder("Stopky: t = 0,0 s");
    plStopWatch.setBorder(title);
    plControls.add(plStopWatch, c);
    plStopWatch.setPreferredSize(new Dimension(220, 50));
    plStopWatch.setLayout(new GridLayout(1, 0));
    plStopWatch.add(btnStart);
    plStopWatch.add(btnReset);

    //add subpanel plVoltage and set its componensts
    c.gridx = 0;
    c.gridy = 1;
    c.weighty = 1.0;
    c.anchor = GridBagConstraints.CENTER;
    title = BorderFactory.createTitledBorder("Naptie = 0 V");
    plVoltage.setBorder(title);
    plControls.add(plVoltage, c);
    plVoltage.setPreferredSize(new Dimension(220, 100));
    plVoltage.setLayout(new BoxLayout(plVoltage, BoxLayout.Y_AXIS));
    slVoltage.setMinorTickSpacing(50);
    slVoltage.setMajorTickSpacing(250);
    slVoltage.setPaintTicks(true);
    slVoltage.setPaintLabels(true);
    slVoltage.setMinimum(0);
    slVoltage.setMaximum(1000);
    slVoltage.setValue(100);
    plVoltage.add(slVoltage);

    plVoltage.add(Box.createRigidArea(new Dimension(220, 5)));

    //add subsubpanel plVoltageButtons and set its componensts
    plVoltageBtns.setLayout(new GridLayout(1, 2));
    plVoltageBtns.add(btnTurnVoltageOn);
    plVoltageBtns.add(btnChangePolarity);

    plVoltage.add(plVoltageBtns);
    plVoltageBtns.setPreferredSize(new Dimension(220, 20));

    //add subpanel plOtherControls and set its componensts
    c.gridx = 0;
    c.gridy = 2;
    c.weighty = 0.0;
    c.anchor = GridBagConstraints.SOUTH;
    title = BorderFactory.createTitledBorder("\u010Ealie ovldanie");
    plOtherControls.setBorder(title);
    plOtherControls.setPreferredSize(new Dimension(220, 50));
    plControls.add(plOtherControls, c);
    plOtherControls.setLayout(new GridLayout(1, 0));
    plOtherControls.add(btnInject);
    plOtherControls.add(btnTurnRaysOn);

    cp.validate();
    cp.repaint();

    //set actions on events
    btnStart.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        stopWatch();
      }
    }
    );
    btnReset.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        t = 0;
        plStopWatch.setBorder(BorderFactory.createTitledBorder(
            "Stopky: t = 0,0 s"));
      }
    }
    );
    slVoltage.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent event) {
        if (voltageon) {
          voltage = slVoltage.getValue();
          drops.setVelocity(voltage);
          plVoltage.setBorder(BorderFactory.createTitledBorder(
              "Naptie = " + slVoltage.getValue() + " V"));
        }
      }
    }
    );
    btnTurnVoltageOn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        if (voltageon) {
          voltageon = false;
          voltage = 0;
          drops.setVelocity(voltage);
          plVoltage.setBorder(BorderFactory.createTitledBorder("Naptie = 0 V"));
          btnTurnVoltageOn.setText("Zapn\u0165");
        }
        else {
          voltageon = true;
          voltage = slVoltage.getValue();
          drops.setVelocity(voltage);
          plVoltage.setBorder(BorderFactory.createTitledBorder("Naptie = " +
              voltage + " V"));
          btnTurnVoltageOn.setText("Vypn\u0165");
        }

      }
    }
    );
    btnChangePolarity.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        if (voltageon) {
          voltage = -1 * voltage;
          drops.setVelocity(voltage);
          plVoltage.setBorder(BorderFactory.createTitledBorder("Naptie = " +
              voltage + " V"));
        }
      }
    }
    );
    btnInject.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        drops.inject();
        drops.setVelocity(voltage);
      }
    }
    );
    btnTurnRaysOn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        if (rayon) {
          rayon = false;
          btnTurnRaysOn.setText("L\u010De zapn\u0165");
        }
        else {
          rayon = true;
          btnTurnRaysOn.setText("L\u010De vypn\u0165");
        }

      }
    }
    );

    panel.setPreferredMinMaxX( -0.75e-3, 0.75e-3);
    panel.setPreferredMinMaxY( -0.75e-3, 0.75e-3);
    panel.setSquareAspect(true);
    panel.setBackground(Color.black);
    panel.enableInspector(false);
    panel.setShowCoordinates(false);
    panel.addDrawable(drops);

    startAnimation();
  }

  public void stopWatch() {
    if (timeRunning) {
      timeRunning = false;
      btnStart.setText("tart");
    }
    else {
      timeRunning = true;
      btnStart.setText("Stop");
    }
  }

  public void startAnimation() {
    if (animationThread != null)return; //already running
    animationThread = new Thread(this);
    animationThread.start();
  }

  public void stopAnimation() {
    Thread tempThread = animationThread; //temporary reference
    animationThread = null; //signal the animation to stop
    if (tempThread != null) {
      try {
        tempThread.interrupt(); //get out of the sleep state
        tempThread.join(); //wait for the thread to die
      }
      catch (InterruptedException e) {}
    }
  }

  public void run() {
    while (animationThread == Thread.currentThread()) {
      try {
        animationThread.sleep(5);
      }
      catch (InterruptedException e) {}

      //set time on stopwatch
      if (timeRunning) {
        t += dt;
        plStopWatch.setBorder(BorderFactory.createTitledBorder("Stopky: t = " +
            decimal.format(t) + " s"));
        plStopWatch.repaint();
      }

      drops.doStep(rayon, voltageon);

      panel.repaint();
    }
  }

  public static void main(String[] args) {
    //set main frame
    JFrame frame = new JFrame();
    frame.setBounds(200, 200, 520, 300);
    frame.setTitle("Millikanov experiment");

    Container cp = frame.getContentPane();
    cp.setLayout(new BorderLayout());

    MillikanExperimentApp applet = new MillikanExperimentApp();
    cp.add(applet);
    applet.init();

    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.setVisible(true);
  }

}
